//
//  MT1STextFieldCellExtention.h
//  TableViewUX
//
//  Created by narita on 2016/08/28.
//  Copyright © 2016年 narita. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// 通常、日本語入力時は、変換を確定しなければ、controlTextDidChangeが呼び出されない。
// プロパティwantsNotificationForMarkedTextがYESにすると、変換途中の場合でもcontrolTextDidChangeが呼び出される。
// 以下のコードはIBでwantsNotificationForMarkedTextを設定できるようにするコード
@interface NSTextFieldCell (MT1STextFieldCellExtention)

@property (readwrite) IBInspectable BOOL wantsNotificationForMarkedText;

@end


@interface NSTextField (MT1STextFieldExtention)

// Textの描画領域とboundsの差の値
@property (readonly) NSSize interSpacing;

// 横固定で、テキストが入る最小の高さの算出
- (CGFloat) calcBoundingHeightWithFixedCurrentWidth;

@end


@interface NSTextView (MT1STextViewExtention)

// NSRangeで示すキャラクターの選択範囲の描画座標での矩形NSRectを返す
- (NSRect) rectForCharacterRange:(NSRange)inRange;


// キャレットの位置が、文字列の先頭にある
- (BOOL) isSelectedRangeTopPosition;

// キャレットの位置が、文字列の末尾にある
- (BOOL) isSelectedRangeLastPosition;



// キャレット位置が、最初の行にある
- (BOOL) isSelectedRangeFirstLine;

// キャレット位置が、最後の行にある
- (BOOL) isSelectedRangeLastLine;


// (1) X座標は与えられた引数の描画座標に一番近い箇所。
// (2) Y座標は最初の行。
// 以上の2つの条件を満たす位置にキャレットを移動する
- (void) selectFirstLineAt:(CGFloat)inX;


// (1) X座標は与えられた引数の描画座標に一番近い箇所。
// (2) Y座標は最後の行。
// 以上の2つの条件を満たす位置にキャレットを移動する
- (void) selectLastLineAt:(CGFloat)inX;

@end