//
//  MT1STextFieldCellExtention.m
//  TableViewUX
//
//  Created by narita on 2016/08/28.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "MT1STextFieldCellExtention.h"

@implementation NSTextField (MT1STextFieldExtention)

// Textの描画領域とboundsの差の値
- (NSSize) interSpacing
{
    NSSize theResult;  
    
    // boundsとdrawingRectの差がoffset値になる。
    NSRect theBounds    = self.bounds;
    NSRect theDrawing   = [self.cell drawingRectForBounds:theBounds];
    
    theResult.width     = theBounds.size.width - theDrawing.size.width;
    theResult.height    = theBounds.size.height - theDrawing.size.height;
    
    return theResult;
}

// 横固定で、テキストが入る最小の高さの算出
- (CGFloat) calcBoundingHeightWithFixedCurrentWidth
{
    NSSize theSize    = self.frame.size;
    
    NSSize theSpacing = [self interSpacing];
    
    // 希望するサイズの補正。ぼんやりした青い枠のサイズを引く。
    theSize.width  = theSize.width - theSpacing.width;
    theSize.height = CGFLOAT_MAX;
    
    // 文字列に合わせた描画領域を算出する
    NSRect theNewRect = [self.attributedStringValue boundingRectWithSize:theSize
                                                                 options:NSStringDrawingUsesLineFragmentOrigin
                                                                 context:nil];
    // 算出した描画領域にぼんやりした青い枠のための補正を追加する。
    theNewRect.size.height += theSpacing.height;

    return theNewRect.size.height; 
}


@end


@implementation NSTextView (MT1STextViewExtention)

// NSRangeで示すキャラクターの選択範囲の描画座標での矩形NSRectを返す
// http://stackoverflow.com/questions/5160094/nspoint-nsrect-from-character-in-nstextview
- (NSRect) rectForCharacterRange:(NSRange)inRange
{    
    NSRange theTextRange       = [self.layoutManager glyphRangeForCharacterRange:inRange 
                                                            actualCharacterRange:NULL];
    NSRect  theLayoutRect      = [self.layoutManager boundingRectForGlyphRange:theTextRange 
                                                               inTextContainer:self.textContainer];
    NSPoint theContainerOrigin = self.textContainerOrigin;
    
    theLayoutRect.origin.x += theContainerOrigin.x;
    theLayoutRect.origin.y += theContainerOrigin.y;
    
    return theLayoutRect;
}

// キャレットの位置が、文字列の先頭にある
- (BOOL) isSelectedRangeTopPosition
{
    NSRange    theRange    = [self selectedRange];
    
    return ( theRange.location == 0 );
}

// キャレットの位置が、文字列の末尾にある
- (BOOL) isSelectedRangeLastPosition
{
    NSRange    theRange    = [self selectedRange];
    NSUInteger theMaxRange = NSMaxRange(theRange);
    NSUInteger theLength   = self.textStorage.length;

    return ( theMaxRange == theLength );
}

// キャレット位置が、最初の行にある
- (BOOL) isSelectedRangeFirstLine
{
    NSRect  theCurrentRect    = [self rectForCharacterRange:self.selectedRange];    
    NSRect  theStartPointRect = [self rectForCharacterRange:NSMakeRange(0, 0)];
    
    // キャレットが、最初の行にあるかの判定
    return( theCurrentRect.origin.y == theStartPointRect.origin.y );
}

// キャレット位置が、最後の行にある
- (BOOL) isSelectedRangeLastLine
{    
    NSRect  theCurrentRect  = [self rectForCharacterRange:self.selectedRange];
    NSRect  theEndPointRect = [self rectForCharacterRange:
                               NSMakeRange(self.textStorage.length, 0)];
    
    // キャレットが、最終行にあるかの判定
    return ( theCurrentRect.origin.y == theEndPointRect.origin.y );
}

// (1) X座標は与えられた引数の描画座標に一番近い箇所。
// (2) Y座標は最初の行。
// 以上の2つの条件を満たす位置にキャレットを移動する
- (void) selectFirstLineAt:(CGFloat)inX
{
    NSPoint    thePoint = NSMakePoint(inX, 0);
    NSUInteger theLocation = [self characterIndexForInsertionAtPoint:thePoint];
    
    self.selectedRange = NSMakeRange(theLocation, 0);    
}


// (1) X座標は与えられた引数の描画座標に一番近い箇所。
// (2) Y座標は最後の行。
// 以上の2つの条件を満たす位置にキャレットを移動する
- (void) selectLastLineAt:(CGFloat)inX
{
    NSRect  theEndPointRect = [self rectForCharacterRange:
                                NSMakeRange(self.textStorage.length, 0)];
    
    NSPoint thePoint = NSMakePoint(inX, theEndPointRect.origin.y);
    
    NSUInteger theLocation = [self characterIndexForInsertionAtPoint:thePoint];
    
    self.selectedRange = NSMakeRange(theLocation, 0);
}

@end
